#include "canvas_listbox.h"

CanvasListbox* canvas_listbox_new (GnomeCanvasGroup *parent, CanvasListboxDesc *desc)
{
	// --------------------------------------------------------------------
	// Creates and initializes new canvas listbox.
	// --------------------------------------------------------------------
	CanvasListbox *lb;
	GnomeCanvasPoints *points;
	
	// --------------------------------------------------------------------
	// Initialization.
	// --------------------------------------------------------------------
	lb = g_new (CanvasListbox, 1);
	lb->desc = desc;
	
	lb->x = 0.0;
	lb->y = 0.0;
	lb->enabled = true;
	lb->hidden = false;
	lb->show_border = true;
	
	lb->empty = true;
	lb->items = g_array_new (false, false, sizeof (CanvasListboxItem*));
	
	// --------------------------------------------------------------------
	// Creation.
	// --------------------------------------------------------------------
	lb->outline = gnome_canvas_item_new (
						parent,
						gnome_canvas_group_get_type (),
						"x", lb->x, "y", lb->y,
						NULL);
	
	points = gnome_canvas_points_new (5);
	points->coords[0] = 0.0;
	points->coords[1] = 0.0;
	points->coords[2] = lb->desc->width;
	points->coords[3] = 0.0;
	points->coords[4] = lb->desc->width;
	points->coords[5] = lb->desc->height;
	points->coords[6] = 0.0;
	points->coords[7] = lb->desc->height;
	points->coords[8] = 0.0;
	points->coords[9] = 0.0;
	
	lb->border = gnome_canvas_item_new (
						GNOME_CANVAS_GROUP (lb->outline),
						gnome_canvas_line_get_type (),
						"points", points,
						"fill_color", lb->desc->border_color,
						"width-units", lb->desc->border_width,
						NULL);
	gnome_canvas_points_free (points);
	if (!lb->show_border) gnome_canvas_item_hide (lb->border);
	
	lb->root_grp = gnome_canvas_item_new (
						GNOME_CANVAS_GROUP (lb->outline),
						gnome_canvas_group_get_type (),
						"x", lb->desc->padding,
						"y", lb->desc->padding,
						NULL);
	
	lb->rect = gnome_canvas_item_new (
						GNOME_CANVAS_GROUP (lb->root_grp),
						gnome_canvas_rect_get_type (),
						"x1", 0.0,
						"y1", 0.0,
						"x2", lb->desc->width - 2*lb->desc->padding,
						"y2", lb->desc->height - 2*lb->desc->padding,
						"fill-color", "#FFFFFE",
						NULL);
						
						
	
	return lb;
}
void canvas_listbox_destroy (CanvasListbox *lbox)
{
	gtk_object_destroy (GTK_OBJECT (lbox->root_grp));
	g_free (lbox->desc);
	g_array_free (lbox->items, true);
	g_free (lbox);
}

// public functions
void canvas_listbox_move_at (CanvasListbox *lbox, double x, double y)
{
	// --------------------------------------------------------------------
	// Moves component to [x,y] coords, relative to its parent
	// --------------------------------------------------------------------
	double x1, y1, x2, y2;
	gnome_canvas_item_get_bounds (lbox->outline, &x1, &y1, &x2, &y2);
	gnome_canvas_item_move (lbox->outline, -x1, -y1);
	gnome_canvas_item_move (lbox->outline, x - lbox->desc->border_width, y - lbox->desc->border_width);
	
	lbox->x = x;
	lbox->y = y;
}
void canvas_listbox_hide (CanvasListbox *lbox)
{
	gnome_canvas_item_hide (lbox->outline);
	lbox->hidden = true;
}
void canvas_listbox_show (CanvasListbox *lbox)
{
	gnome_canvas_item_show (lbox->outline);
	lbox->hidden = false;
}
void canvas_listbox_insert (CanvasListbox *lbox, char* text, char* data, int id)
{
	// --------------------------------------------------------------------
	// This function creates gnome canvas text item and adds it to listbox.
	// --------------------------------------------------------------------
	CanvasListboxItem* item;
	
	// create item
	item = g_new (CanvasListboxItem, 1);
	item->item = gnome_canvas_item_new (
						GNOME_CANVAS_GROUP (lbox->root_grp),
						gnome_canvas_text_get_type (),
						"x", 0.0, "y", 0.0,
						"text", text,
						"fill-color", lbox->desc->text_color,
						"size", lbox->desc->text_size,
						"anchor", GTK_ANCHOR_NORTH_WEST,
						NULL);
	item->text = g_strdup_printf ("%s", text);
	item->data = g_strdup_printf ("%s", data);
	item->id = id;
	
	// insert item into vector
	//(g_utf8_collate (g_utf8_casefold (lbox->items[index]->text, -1), g_utf8_casefold (text, -1)) <= 0)
	g_array_append_val (lbox->items, item);
	lbox->empty = false;
	canvas_listbox_reorder (lbox);
}
CanvasListboxItem* canvas_listbox_get_item_at (CanvasListbox *lbox, double x, double y)
{
	// --------------------------------------------------------------------
	// Return item at coords [x,y].
	// Coords are relative to lbox->root_grp.
	// --------------------------------------------------------------------
	guint i;
	double x1, y1, x2, y2;
	GnomeCanvasItem *item;
	CanvasListboxItem *lstbox_item;
	
	if (lbox->items->len == 0) return NULL;
	for (i = 0; i < lbox->items->len; i++)
	{
		lstbox_item = g_array_index (lbox->items, CanvasListboxItem*, i);
		item = lstbox_item->item;
		gnome_canvas_item_get_bounds (item, &x1, &y1, &x2, &y2);
		if ((x >= x1) && (x <= x2) && (y >= y1) && (y <= y2))
			return lstbox_item;
	}
	
	return NULL;
}

// properties
gboolean canvas_listbox_is_empty (CanvasListbox *lbox)
{
	// --------------------------------------------------------------------
	// Query listbox on its emptiness.
	// --------------------------------------------------------------------
	if (lbox->empty)
		return true;
	else
		return false;
}

// private functions
void canvas_listbox_reorder (CanvasListbox *lbox)
{
	// --------------------------------------------------------------------
	// Reorders listbox so that its elements are listed in the correct
	// order on the canvas.
	// --------------------------------------------------------------------
	guint i;
	CanvasListboxItem* lb_item;
	GnomeCanvasItem* item;
	double x1, y1, x2, y2;
	double last_y = 0.0;
	
	if (lbox->items->len > 0)
		for (i = 0; i < lbox->items->len; i++)
		{
			lb_item = g_array_index (lbox->items, CanvasListboxItem*, i);
			item = lb_item->item;
			gnome_canvas_item_get_bounds (item, &x1, &y1, &x2, &y2);
			canvas_item_move_at (lb_item->item, 0.0, last_y);
			last_y = y2;
		}
}

// other
void canvas_item_move_at (GnomeCanvasItem* item, double x, double y)
{
	// --------------------------------------------------------------------
	// Moves item on [x,y] coords.
	// --------------------------------------------------------------------
	double x1, x2, y1, y2;
	gnome_canvas_item_get_bounds (item, &x1, &y1, &x2, &y2);
	gnome_canvas_item_move (item, -x1, -y1);
	gnome_canvas_item_move (item, x, y);
}



